//**********************************************************************************************************************
//
//  FMColorCorrectionWithOvalMask.fs
//
//  Author:		Peter Baumgartner, peter@baumgartner.com
//  Copyright:	©2011-2013 by IMAGINE GbR. All rights reserved.
//	Abstract:	Applies color correction and a oval mask
//
//**********************************************************************************************************************


// Color correction...

uniform sampler2DRect uTexture;
uniform mat4 uColorCorrectionMatrix;
uniform vec4 uBlackLevelM;
uniform vec4 uBlackLevelT;
uniform vec4 uGamma;
uniform float uOpacity;
uniform vec4 uMixColor;

// Mask...

varying vec2 vMaskLocation;
uniform vec2 uInnerDim;
uniform vec2 uOuterDim;
uniform float uRoundness;
uniform float uMaskOpacityOffset;
uniform float uMaskOpacityFactor;


//----------------------------------------------------------------------------------------------------------------------


// This code was written by Maximillian Christ. Some useful info can be found at:
// http://de.wikipedia.org/wiki/Superellipse
// http://www.yaldex.com/open-gl/ch12lev1sec4.html

float shape(vec2 pos)
{
	vec2 abspos = abs(pos);
	
	// TODO: move this calculation to the vertex shader or to a uniform vec2 roundness
	vec4 exp1 = vec4(2.0 / uRoundness);
	vec2 exp2 = vec2(-uRoundness / 2.0);
	
	vec4 ioxy = vec4(abspos.xy * uInnerDim.yx, abspos.xy * uOuterDim.yx);
	
	vec4 p1 = pow(ioxy, exp1);
	vec2 iop = p1.xz + p1.yw;
	
	vec2 iox = vec2(uInnerDim.x,uOuterDim.x);
	vec2 ioy = vec2(uInnerDim.y,uOuterDim.y);
	
	vec2 io = iox * ioy * pow(iop, exp2);
	return 1.0 - smoothstep(io.x, io.y, 1.0);
}


//----------------------------------------------------------------------------------------------------------------------


void main()
{
	vec3 pos = vec3(vMaskLocation.xy,0.0);

	// Calculate the alpha value for the mask...
	
	float mask = shape(pos.xy);
	float opacity = uMaskOpacityOffset + uMaskOpacityFactor * mask;
	opacity *= uOpacity;
	
	// Get pixel color from texture...
	
	vec2 xy		= gl_TexCoord[0].xy;
	vec4 color	= texture2DRect(uTexture,xy);
	float alpha = color.a * opacity;

	// Apply color correction (tint and saturation via matrix)...
	
	color = uColorCorrectionMatrix * color;
	color = clamp(color,0.0,1.0);

	// Correct black level (linear equation that boost contrast)...
	
	color = (uBlackLevelM * color) + uBlackLevelT;
	color = clamp(color,0.0,1.0);

	// Apply gamma via pow function...
	
	color = pow(color,uGamma);
	
    // Mix with an optional color...
    
    color.rgb = (1.0-uMixColor.a) * color.rgb + uMixColor.a * uMixColor.rgb;
    
	// Pre-multiply the color with pixel alpha and global texture opacity (because all drawing is 
	// done with pre-multiplied blending GL_ONE,GL_ONE_MINUS_SRC_ALPHA)...
	
	color.rgb *= opacity;
	color.a = alpha;
	
	// Return corrected color...
	
	gl_FragColor = color;
}


//----------------------------------------------------------------------------------------------------------------------
