//**********************************************************************************************************************
//
//  FMColorCorrectionWithOvalMask.vs
//
//  Author:		Peter Baumgartner, peter@baumgartner.com
//  Copyright:	©2011-2012 by IMAGINE GbR. All rights reserved.
//	Abstract:	Applies color correction and a rect mask
//
//**********************************************************************************************************************


// Color correction...

uniform sampler2DRect uTexture;
uniform mat4 uColorCorrectionMatrix;
uniform vec4 uBlackLevelM;
uniform vec4 uBlackLevelT;
uniform vec4 uGamma;
uniform float uOpacity;
uniform vec4 uMixColor;

// Mask...

varying vec2 vMaskLocation;

uniform float uLeft0;
uniform float uLeft1;
uniform float uRight1;
uniform float uRight0;

uniform float uTop0;
uniform float uTop1;
uniform float uBottom1;
uniform float uBottom0;

uniform float uMaskOpacityOffset;
uniform float uMaskOpacityFactor;


//----------------------------------------------------------------------------------------------------------------------


void main()
{
	// Create the mask by modifying the opacity locally. vMaskLocation is the fragment coordinate relative to the 
	// mask, already rotated back to be parallel to the stage, so we simply need to calculate four smoothsteps for 
	// the edges and multiply them...
	
    float maskLeft		= smoothstep(uLeft0,uLeft1,vMaskLocation.x);
    float maskRight		= 1.0 - smoothstep(uRight1,uRight0,vMaskLocation.x);
    float maskTop		= 1.0 - smoothstep(uTop1,uTop0,vMaskLocation.y);
    float maskBottom	= smoothstep(uBottom0,uBottom1,vMaskLocation.y);
	float mask			= maskLeft * maskRight * maskTop * maskBottom;
	
	float opacity		= uMaskOpacityOffset + uMaskOpacityFactor * mask;
	opacity *= uOpacity;
	
	// Get pixel color from texture...
	
	vec2 xy		= gl_TexCoord[0].xy;
	vec4 color	= texture2DRect(uTexture,xy);
	float alpha = color.a * opacity;
	
	// Apply color correction (tint and saturation via matrix)...
	
	color = uColorCorrectionMatrix * color;
	color = clamp(color,0.0,1.0);
	
	// Correct black level (linear equation that boost contrast)...
	
	color = (uBlackLevelM * color) + uBlackLevelT;
	color = clamp(color,0.0,1.0);

	// Apply gamma via pow function...
	
	color = pow(color,uGamma);
	
    // Mix with an optional color...
    
    color.rgb = (1.0-uMixColor.a) * color.rgb + uMixColor.a * uMixColor.rgb;
    
	// Pre-multiply the color with pixel alpha and global texture opacity (because all drawing is 
	// done with pre-multiplied blending GL_ONE,GL_ONE_MINUS_SRC_ALPHA)...
	
	color.rgb *= opacity;
	color.a = alpha;

	// Return corrected color...
	
	gl_FragColor = color;
}


//----------------------------------------------------------------------------------------------------------------------
